# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from cms.models import CMSPlugin
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class HtmlSitemapPluginConf(CMSPlugin):
    min_depth = models.PositiveIntegerField(
        verbose_name=_('Minimum depth'), default=0)
    max_depth = models.PositiveIntegerField(
        verbose_name=_('Maximum depth'), blank=True, null=True)
    in_navigation = models.NullBooleanField(
        verbose_name=_('In navigation'), default=None)

    class Meta:
        verbose_name = _('HTML Sitemap plugin configuration')
        verbose_name_plural = _('HTML Sitemap plugin configurations')

    def __str__(self):
        return 'Django-CMS HTML Sitemap #{0}'.format(self.pk)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0011_auto_20150419_1006'),
    ]

    operations = [
        migrations.CreateModel(
            name='HtmlSitemapPluginConf',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('min_depth', models.PositiveIntegerField(default=0, verbose_name='Minimum depth')),
                ('max_depth', models.PositiveIntegerField(null=True, verbose_name='Maximum depth', blank=True)),
                ('in_navigation', models.NullBooleanField(default=None, verbose_name='In navigation')),
            ],
            options={
                'verbose_name': 'HTML Sitemap plugin configuration',
                'verbose_name_plural': 'HTML Sitemap plugin configurations',
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# -*- coding: utf-8 -*-

# Standard library imports
import os

# Third party imports
from django import VERSION as DJANGO_VERSION
from django.conf import global_settings as default_settings

# Local application / specific library imports


TEST_ROOT = os.path.abspath(os.path.dirname(__file__))

# Helper function to extract absolute path
location = lambda x: os.path.join(TEST_ROOT, x)


class DisableMigrations(object):
    def __contains__(self, item):
        return True

    def __getitem__(self, item):
        return 'notmigrations'


DEBUG = False
TEMPLATE_DEBUG = False

SECRET_KEY = 'key'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:'
    },
}

if DJANGO_VERSION >= (1, 8):
    TEMPLATES = [
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': (
                location('_testsite/templates'),
            ),
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': [
                    'django.contrib.auth.context_processors.auth',
                    'django.template.context_processors.debug',
                    'django.template.context_processors.i18n',
                    'django.template.context_processors.media',
                    'django.template.context_processors.request',
                    'django.template.context_processors.static',
                    'django.template.context_processors.tz',
                    'django.contrib.messages.context_processors.messages',
                    'cms.context_processors.cms_settings',
                    'sekizai.context_processors.sekizai',
                ],
            },
        },
    ]
else:
    TEMPLATE_CONTEXT_PROCESSORS = default_settings.TEMPLATE_CONTEXT_PROCESSORS + (
        'django.core.context_processors.request',
        'cms.context_processors.cms_settings',
    )
    TEMPLATE_DIRS = (
        location('_testsite/templates'),
    )

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sessions',
    'django.contrib.sites',

    'cms',
    'treebeard',
    'menus',
    'sekizai',
    'djangocms_admin_style',

    'djangocms_htmlsitemap',
    'tests',
] + ['django.contrib.admin', ]

MIGRATION_MODULES = DisableMigrations()
TEST_RUNNER = 'django.test.runner.DiscoverRunner'  # Hide checks

CMS_TEMPLATES = (
    ('index.html', 'Index'),
    ('simple.html', 'Simple'),
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'cms.middleware.user.CurrentUserMiddleware',
    'cms.middleware.page.CurrentPageMiddleware',
    'cms.middleware.toolbar.ToolbarMiddleware',
    'cms.middleware.language.LanguageCookieMiddleware',
)

MEDIA_ROOT = '/media/'
STATIC_URL = '/static/'

USE_TZ = True
LANGUAGE_CODE = 'fr'
LANGUAGES = (
    ('fr', 'Français'),
    ('en', 'English'),
)

SITE_ID = 1

ROOT_URLCONF = 'tests._testsite.urls'

# Setting this explicitly prevents Django 1.7+ from showing a
# warning regarding a changed default test runner. The test
# suite is run with py.test, so it does not matter.
SILENCED_SYSTEM_CHECKS = ['1_6.W001']



from os.path import abspath
from os.path import dirname
from os.path import join
from setuptools import find_packages
from setuptools import setup


def read_relative_file(filename):
    """
    Returns contents of the given file, whose path is supposed relative
    to this module.
    """
    with open(join(dirname(abspath(__file__)), filename)) as f:
        return f.read()


setup(
    name='djangocms-htmlsitemap',
    version=read_relative_file('VERSION').strip(),
    author='Kapt',
    author_email='dev@kapt.mobi',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/kapt-labs/djangocms-htmlsitemap',
    license='BSD',
    description='A Django CMS plugin for building HTML sitemaps showing organized lists of CMS pages.',
    long_description=read_relative_file('README.rst'),
    zip_safe=False,
    install_requires=[
        'django>=1.6',
        'django-cms>=3.1',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf.urls import include
from django.conf.urls import patterns
from django.conf.urls import url
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

# Local application / specific library imports


admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^', include('cms.urls')),
)

urlpatterns += staticfiles_urlpatterns()



# -*- coding:utf-8 -*-

from __future__ import unicode_literals

from cms.api import add_plugin
from cms.api import create_page
from cms.models import Placeholder
from django.test.client import RequestFactory
from django.utils.html import strip_spaces_between_tags
import pytest

from djangocms_htmlsitemap import cms_plugins


@pytest.mark.django_db
class TestHtmlSitemapPlugin(object):
    @pytest.fixture(autouse=True)
    def setup_cms(self):
        self.request_factory = RequestFactory()

        # Creates a basic tree of CMS pages
        self.index_page = create_page('Index', 'index.html', 'en', published=True, in_navigation=True)  # noq
        self.depth2_page1 = create_page(
            'Depth 2 page 1', 'simple.html', 'en', in_navigation=True, published=True, parent=self.index_page)
        self.depth2_page2 = create_page(
            'Depth 2 page 2', 'simple.html', 'en', in_navigation=False, published=True, parent=self.index_page)
        self.depth3_page1 = create_page(
            'Depth 3 page 1', 'simple.html', 'en', in_navigation=False, published=True, parent=self.depth2_page2)
        self.depth3_page2 = create_page(
            'Depth 3 page 2', 'simple.html', 'en', in_navigation=False, published=True, parent=self.depth2_page2)
        self.depth2_page3 = create_page(
            'Depth 2 page 3', 'simple.html', 'en', in_navigation=False, published=True, parent=self.index_page)
        self.depth2_page4 = create_page(
            'Depth 2 page 4', 'simple.html', 'en', in_navigation=False, published=True, parent=self.index_page)
        self.depth3_page3 = create_page(
            'Depth 3 page 3', 'simple.html', 'en', in_navigation=False, published=True, parent=self.depth2_page4)

    def test_can_render_a_simple_tree_of_cms_pages(self):
        # Setup
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            cms_plugins.HtmlSitemapPlugin,
            'fr',
        )

        # Run
        html = model_instance.render_plugin({})
        html = strip_spaces_between_tags(html)

        # Check
        assert html.strip() == strip_spaces_between_tags(
            """
                <div id="sitemap">
                    <ul>
                        <li>
                            <a href="/" title="Index">Index</a>
                            <ul>
                                <li><a href="/depth-2-page-1/" title="Depth 2 page 1">Depth 2 page 1</a></li>
                                <li>
                                    <a href="/depth-2-page-2/" title="Depth 2 page 2">Depth 2 page 2</a>
                                    <ul>
                                        <li><a href="/depth-2-page-2/depth-3-page-1/" title="Depth 3 page 1">Depth 3 page 1</a></li>
                                        <li><a href="/depth-2-page-2/depth-3-page-2/" title="Depth 3 page 2">Depth 3 page 2</a></li>
                                    </ul>
                                </li>
                                <li><a href="/depth-2-page-3/" title="Depth 2 page 3">Depth 2 page 3</a></li>
                                <li>
                                    <a href="/depth-2-page-4/" title="Depth 2 page 4">Depth 2 page 4</a>
                                    <ul>
                                        <li><a href="/depth-2-page-4/depth-3-page-3/" title="Depth 3 page 3">Depth 3 page 3</a></li>
                                    </ul>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </div>
            """).strip()

    def test_can_render_a_simple_tree_of_cms_pages_from_a_minimum_depth(self):
        # Setup
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            cms_plugins.HtmlSitemapPlugin,
            'fr',
            min_depth=2,
        )

        # Run
        html = model_instance.render_plugin({})
        html = strip_spaces_between_tags(html)

        # Check
        assert html.strip() == strip_spaces_between_tags(
            """
                <div id="sitemap">
                    <ul>
                        <li><a href="/depth-2-page-1/" title="Depth 2 page 1">Depth 2 page 1</a></li>
                        <li>
                            <a href="/depth-2-page-2/" title="Depth 2 page 2">Depth 2 page 2</a>
                            <ul>
                                <li><a href="/depth-2-page-2/depth-3-page-1/" title="Depth 3 page 1">Depth 3 page 1</a></li>
                                <li><a href="/depth-2-page-2/depth-3-page-2/" title="Depth 3 page 2">Depth 3 page 2</a></li>
                            </ul>
                        </li>
                        <li><a href="/depth-2-page-3/" title="Depth 2 page 3">Depth 2 page 3</a></li>
                        <li>
                            <a href="/depth-2-page-4/" title="Depth 2 page 4">Depth 2 page 4</a>
                            <ul>
                                <li><a href="/depth-2-page-4/depth-3-page-3/" title="Depth 3 page 3">Depth 3 page 3</a></li>
                            </ul>
                        </li>
                    </ul>
                </div>
            """).strip()

    def test_can_render_a_simple_tree_of_cms_pages_to_a_maximum_depth(self):
        # Setup
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            cms_plugins.HtmlSitemapPlugin,
            'fr',
            max_depth=2,
        )

        # Run
        html = model_instance.render_plugin({})
        html = strip_spaces_between_tags(html)

        # Check
        assert html.strip() == strip_spaces_between_tags(
            """
                <div id="sitemap">
                    <ul>
                        <li>
                            <a href="/" title="Index">Index</a>
                            <ul>
                                <li><a href="/depth-2-page-1/" title="Depth 2 page 1">Depth 2 page 1</a></li>
                                <li>
                                    <a href="/depth-2-page-2/" title="Depth 2 page 2">Depth 2 page 2</a>
                                </li>
                                <li><a href="/depth-2-page-3/" title="Depth 2 page 3">Depth 2 page 3</a></li>
                                <li>
                                    <a href="/depth-2-page-4/" title="Depth 2 page 4">Depth 2 page 4</a>
                                </li>
                            </ul>
                        </li>
                    </ul>
                </div>
            """).strip()

    def test_can_render_a_simple_tree_of_cms_pages_from_a_minimum_depth_to_a_maximum_depth(self):
        # Setup
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            cms_plugins.HtmlSitemapPlugin,
            'fr',
            min_depth=2,
            max_depth=2,
        )

        # Run
        html = model_instance.render_plugin({})
        html = strip_spaces_between_tags(html)

        # Check
        assert html.strip() == strip_spaces_between_tags(
            """
                <div id="sitemap">
                    <ul>
                        <li><a href="/depth-2-page-1/" title="Depth 2 page 1">Depth 2 page 1</a></li>
                        <li>
                            <a href="/depth-2-page-2/" title="Depth 2 page 2">Depth 2 page 2</a>
                        </li>
                        <li><a href="/depth-2-page-3/" title="Depth 2 page 3">Depth 2 page 3</a></li>
                        <li>
                            <a href="/depth-2-page-4/" title="Depth 2 page 4">Depth 2 page 4</a>
                        </li>
                    </ul>
                </div>
            """).strip()

    def test_can_render_a_simple_tree_of_cms_pages_that_are_in_navigation(self):
        # Setup
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            cms_plugins.HtmlSitemapPlugin,
            'fr',
            in_navigation=True,
        )

        # Run
        html = model_instance.render_plugin({})
        html = strip_spaces_between_tags(html)

        # Check
        assert html.strip() == strip_spaces_between_tags(
            """
                <div id="sitemap">
                    <ul>
                        <li>
                            <a href="/" title="Index">Index</a>
                            <ul>
                                <li><a href="/depth-2-page-1/" title="Depth 2 page 1">Depth 2 page 1</a></li>
                            </ul>
                        </li>
                    </ul>
                </div>
            """).strip()

    def test_can_render_a_simple_tree_of_cms_pages_that_are_not_in_navigation(self):
        # Setup
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            cms_plugins.HtmlSitemapPlugin,
            'fr',
            in_navigation=False,
        )

        # Run
        html = model_instance.render_plugin({})
        html = strip_spaces_between_tags(html)

        # Check
        assert html.strip() == strip_spaces_between_tags(
            """
                <div id="sitemap">
                    <ul>
                        <li>
                            <a href="/depth-2-page-2/" title="Depth 2 page 2">Depth 2 page 2</a>
                            <ul>
                                <li><a href="/depth-2-page-2/depth-3-page-1/" title="Depth 3 page 1">Depth 3 page 1</a></li>
                                <li><a href="/depth-2-page-2/depth-3-page-2/" title="Depth 3 page 2">Depth 3 page 2</a></li>
                            </ul>
                        </li>
                        <li><a href="/depth-2-page-3/" title="Depth 2 page 3">Depth 2 page 3</a></li>
                        <li>
                            <a href="/depth-2-page-4/" title="Depth 2 page 4">Depth 2 page 4</a>
                            <ul>
                                <li><a href="/depth-2-page-4/depth-3-page-3/" title="Depth 3 page 3">Depth 3 page 3</a></li>
                            </ul>
                        </li>
                    </ul>
                </div>
            """).strip()



# -*- coding:utf-8 -*-

from __future__ import unicode_literals

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from cms.models.pagemodel import Page
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _

from .models import HtmlSitemapPluginConf


class HtmlSitemapPlugin(CMSPluginBase):
    model = HtmlSitemapPluginConf
    name = _('HTML Sitemap')

    render_template = 'djangocms_htmlsitemap/sitemap.html'

    def render(self, context, instance, placeholder):
        site = Site.objects.get_current()
        pages = Page.objects.public().published(site=site).order_by('path') \
            .filter(depth__gte=instance.min_depth).distinct()
        if instance.max_depth:
            pages = pages.filter(depth__lte=instance.max_depth)
        if instance.in_navigation is not None:
            pages = pages.filter(in_navigation=instance.in_navigation)

        context['instance'] = instance
        context['pages'] = pages
        context['annotated_pages'] = Page.get_annotated_list_qs(pages)

        return context


plugin_pool.register_plugin(HtmlSitemapPlugin)
